package swarmnet;

import java.util.ArrayList;
import java.util.List;

public class Node {

	private int x, y;
	private List<Packet> packets;
	private List<Node> neighbourNodes;
	private int hopsToSink;
	private int numVisits;

	public Node(int x, int y) {

		this.x = x;
		this.y = y;
		packets = new ArrayList<Packet>();
		neighbourNodes = new ArrayList<Node>();
		hopsToSink = -1;
		numVisits = 0;
	}

	public boolean isSink() {
		return (hopsToSink == 0);
	}


	public String info() {
		return "(" + x + ", " + y + ")" + " hops to sink = " + hopsToSink
				+ " num packets = " + packets.size();
	}

	public void setHopsToSink(int h) {
		hopsToSink = h;
	}

	public int getHopsToSink() {
		return hopsToSink;
	}

	static double dist(Node n1, Node n2) {

		double delX = n1.getX() - n2.getX();
		double delY = n1.getY() - n2.getY();
		return Math.sqrt(delX * delX + delY * delY);
	}

	void setNeighbours(List<Node> nodes, double neighbourDistance) {

		for (Node m : nodes) {

			if (this.equals(m))
				continue;

			if (dist(this, m) < neighbourDistance) {
				neighbourNodes.add(m);
			}
		}

	}

	public void addPacket(Packet p) {
		packets.add(p);
		++numVisits;
	}

	public List<Packet> getPackets() {
		return packets;
	}

	public int numPackets() {
		return packets.size();
	}

	public void removePacket(Packet p) {
		packets.remove(p);
	}

	public void removePacket(int i) {
		packets.remove(i);
	}

	public Packet getPacket(int i) {

		if (i < packets.size() && i >= 0)
			return packets.get(i);
		else
			return null;
	}

	public boolean isEmpty() {
		return packets.isEmpty();
	}

	public int getX() {
		return x;
	}

	public int getY() {
		return y;
	}

	public void addNeighbours(List<Node> nodeList) {
		neighbourNodes = nodeList;
	}

	public List<Node> getNeighbours() {
		return neighbourNodes;
	}

	public int numNeighbours() {
		return neighbourNodes.size();
	}
	
	public int getNumVisits(){
		return numVisits;
	}
}
