package swarmnet;

import java.awt.event.KeyEvent;
import java.util.List;

import processing.core.PApplet;
import processing.core.PFont;

public class SwarmNetAnimator extends PApplet {

    private static final long serialVersionUID = 1L;

    private PFont font;
    private boolean stop;
    private long configSeed, randomSeed;

    private int algo;
    private int numAlgos;

    //	private SwarmNet swarmNet;
    private SwarmNet2 swarmNet;

    public void setup() {

        size(screenWidth / 2, screenWidth / 2);

        algo = 1;
        numAlgos = 2;
        randomSeed = 21122010;
        configSeed = 32498572;
        //swarmNet = new SwarmNet(width, height, configSeed, randomSeed, algo);
        swarmNet = new SwarmNet2(width, height, configSeed, randomSeed);

        font = loadFont("Verdana-48.vlw");
        textFont(font, 12);

        ellipseMode(CENTER);
        rectMode(CENTER);
        smooth();
        frameRate(5);
        background(0);
        stop = false;
    }

    public void draw() {

        if (stop)
            return;

        List<Node> nodes = swarmNet.getNodes();
        Node sink = swarmNet.getSink();

        stop = swarmNet.iterate();

        background(0);

        for (Node node : nodes) {

            noStroke();
            if (node.getNumVisits() == 0)
                fill(255);
            else
                fill(255, 0, 255);
            rect(node.getX(), node.getY(), 5, 5);
            // text(node.getHopsToSink(), node.getX() + 5, node.getY());

            noStroke();
            int shade = 255 / swarmNet.getMaxQueue();
            fill(0, 255, 0, shade * node.numPackets());
            rect(node.getX(), node.getY(), 20, 20);

            for (Node nn : node.getNeighbours()) {
                stroke(0, 0, 255, 64);
                line(nn.getX(), nn.getY(), node.getX(), node.getY());

            }
        }
        noStroke();
        fill(255, 0, 0);
        rect(sink.getX(), sink.getY(), 5, 5);

        fill(255);
        text(swarmNet.report(), 5, height - 5);

    }

    public void keyPressed() {

        if (key == KeyEvent.VK_SPACE) {
            stop = !stop;
        } else if (key == 'r') { // New network

            stop = true;
            ++randomSeed;
            ++configSeed;
            //			swarmNet = new SwarmNet(width, height, configSeed, randomSeed, algo);
            swarmNet = new SwarmNet2(width, height, configSeed, randomSeed);
            stop = false;

        } else if (key == 's') { // Keep network, different sequence of random nums

            stop = true;
            ++randomSeed;
            //			swarmNet = new SwarmNet(width, height, configSeed, randomSeed, algo);
            swarmNet = new SwarmNet2(width, height, configSeed, randomSeed);
            stop = false;

        } else if (key == 'a') { // Run same simulation with different algorithm

            stop = true;
            ++algo;
            algo %= numAlgos;
            println("Algorithm: " + algo);
            //			swarmNet = new SwarmNet(width, height, configSeed, randomSeed, algo);
            swarmNet = new SwarmNet2(width, height, configSeed, randomSeed);
            stop = false;
        }
    }

    public void exit(String s) {

        println(s);
        System.exit(0);
    }

    public static void main(String[] args) {

        //PApplet.main(new String[] { "--present", "swarmnet.SwarmNetAnimator" });
        PApplet.main(new String[] { "swarmnet.SwarmNetAnimator" });
    }
}
